package org.processmining.acceptingpetrinetclassicalreductor.algorithms;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinet.utils.PetriNetUtils;
import org.processmining.acceptingpetrinetclassicalreductor.parameters.ReduceUsingMurataRulesParameters;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.petrinet.reduction.Murata;
import org.processmining.plugins.petrinet.reduction.MurataInput;
import org.processmining.plugins.petrinet.reduction.MurataOutput;
import org.processmining.plugins.petrinet.reduction.MurataParameters;

public class ReduceUsingMurataRulesAlgorithm {

	public AcceptingPetriNet apply(PluginContext context, AcceptingPetriNet net,
			ReduceUsingMurataRulesParameters parameters) {
		AcceptingPetriNet reducedNet = AcceptingPetriNetFactory.createAcceptingPetriNet();

		Murata murata = new Murata();
		MurataInput input = new MurataInput(net.getNet(), net.getInitialMarking());
		// Make all visible transitions sacred.
		input.setVisibleSacred(net.getNet());
		// Make all places involved in the initial marking scared.
		for (Place place : net.getInitialMarking().baseSet()) {
			input.addSacred(place);
		}
		// Make all places involved in a final marking scared.
		for (Marking finalMarking : net.getFinalMarkings()) {
			for (Place place : finalMarking.baseSet()) {
				input.addSacred(place);
			}
		}
		try {
			// Try to reduce non-sacred place and non-scared transitions.
			MurataParameters params = new MurataParameters();
			/*
			 * Only apply FPT rule in case both nodes are not sacred, that is, if both transitions are invisible.
			 */
			params.setAllowFPTSacredNode(!parameters.isRetainBehavior());
			MurataOutput output = murata.run(null, input, params);
			reducedNet.init(output.getNet());
			reducedNet.setInitialMarking(PetriNetUtils.apply(net.getInitialMarking(), output.getPlaceMapping()));
			reducedNet.setFinalMarkings(PetriNetUtils.apply(net.getFinalMarkings(), output.getPlaceMapping()));
		} catch (ConnectionCannotBeObtained e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return reducedNet;
	}

}
