package org.processmining.acceptingpetrinetclassicalreductor.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinetclassicalreductor.algorithms.ReduceUsingMurataRulesAlgorithm;
import org.processmining.acceptingpetrinetclassicalreductor.connections.ReduceUsingMurataRulesConnection;
import org.processmining.acceptingpetrinetclassicalreductor.help.ReduceUsingMurataRulesHelp;
import org.processmining.acceptingpetrinetclassicalreductor.parameters.ReduceUsingMurataRulesParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "Reduce Using Murata Rules", parameterLabels = { "Accepting Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class }, help = ReduceUsingMurataRulesHelp.TEXT)
public class ReduceUsingMurataRulesPlugin extends ReduceUsingMurataRulesAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Reduce Using Murata Rules, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runDefault(PluginContext context, AcceptingPetriNet net) {
		ReduceUsingMurataRulesParameters parameters = new ReduceUsingMurataRulesParameters();
		return reducePrivateConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet reduceDefault(PluginContext context, AcceptingPetriNet net) {
		return runDefault(context, net);
	}

	@PluginVariant(variantLabel = "Reduce Using Murata Rules, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet run(PluginContext context, AcceptingPetriNet net,
			ReduceUsingMurataRulesParameters parameters) {
		return reducePrivateConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public AcceptingPetriNet reduceParameters(PluginContext context, AcceptingPetriNet net,
			ReduceUsingMurataRulesParameters parameters) {
		return run(context, net, parameters);
	}

	private AcceptingPetriNet reducePrivateConnection(PluginContext context, AcceptingPetriNet net,
			ReduceUsingMurataRulesParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<ReduceUsingMurataRulesConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(ReduceUsingMurataRulesConnection.class,
						context, net);
				for (ReduceUsingMurataRulesConnection connection : connections) {
					if (connection.getObjectWithRole(ReduceUsingMurataRulesConnection.NET).equals(net)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(ReduceUsingMurataRulesConnection.REDUCEDNET);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNet reducedNet = apply(context, net, parameters);
		if (parameters.isTryConnections()) {
			context.getProvidedObjectManager().createProvidedObject(
					"Initial marking for " + reducedNet.getNet().getLabel(), reducedNet.getInitialMarking(),
					Marking.class, context);
			context.addConnection(new InitialMarkingConnection(reducedNet.getNet(), reducedNet.getInitialMarking()));
			context.getConnectionManager().addConnection(
					new ReduceUsingMurataRulesConnection(net, reducedNet, parameters));
		}
		return reducedNet;
	}

}
